@@ -39,14 +39,11 @@ __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2005-2009 Sun Microsystems Inc."
 __license__   = "LGPL"
 
-import imp
-import importlib
 import pyatspi
 
 from . import braille_generator
 from . import debug
 from . import event_manager
-from . import flat_review
 from . import formatting
 from . import label_inference
 from . import keybindings
@@ -55,6 +52,7 @@ from . import script_manager
 from . import script_utilities
 from . import settings
 from . import settings_manager
+from . import sound_generator
 from . import speech_generator
 from . import structural_navigation
 from . import where_am_I
@@ -83,8 +81,8 @@ class Script:
             try:
                 self.name = self.app.name
             except (LookupError, RuntimeError):
-                msg = 'script.__init__: Error trying to get app.name'
-                debug.println(debug.LEVEL_FINE, msg)
+                msg = 'ERROR: Could not get name of script app %s'
+                debug.println(debug.LEVEL_INFO, msg, True)
                 self.name = "default"
         else:
             self.name = "default"
@@ -100,6 +98,10 @@ class Script:
         self.utilities = self.getUtilities()
         self.labelInference = self.getLabelInference()
         self.structuralNavigation = self.getStructuralNavigation()
+        self.caretNavigation = self.getCaretNavigation()
+        self.bookmarks = self.getBookmarks()
+        self.liveRegionManager = self.getLiveRegionManager()
+
         self.chat = self.getChat()
         self.inputEventHandlers = {}
         self.pointOfReference = {}
@@ -109,21 +111,19 @@ class Script:
 
         self.formatting = self.getFormatting()
         self.brailleGenerator = self.getBrailleGenerator()
+        self.soundGenerator = self.getSoundGenerator()
         self.speechGenerator = self.getSpeechGenerator()
         self.generatorCache = {}
         self.eventCache = {}
         self.whereAmI = self.getWhereAmI()
-        self.bookmarks = self.getBookmarks()
         self.spellcheck = self.getSpellCheck()
-        self.voices = settings.voices
         self.tutorialGenerator = self.getTutorialGenerator()
 
-        self.flatReviewContextClass = flat_review.Context
-
         self.findCommandRun = False
         self._lastCommandWasStructNav = False
 
-        debug.println(debug.LEVEL_FINE, "NEW SCRIPT: %s" % self.name)
+        msg = 'SCRIPT: %s initialized' % self.name
+        debug.println(debug.LEVEL_INFO, msg, True)
 
     def getListeners(self):
         """Sets up the AT-SPI event listeners for this script.
@@ -193,7 +193,7 @@ class Script:
         braille.py) that match the given inputEventHandler passed.
         """
         return [command
-                for command, handler in list(self.brailleBindings.items())
+                for command, handler in self.brailleBindings.items()
                 if inputEventHandler == handler]
 
     def getFormatting(self):
@@ -205,6 +205,10 @@ class Script:
         """
         return braille_generator.BrailleGenerator(self)
 
+    def getSoundGenerator(self):
+        """Returns the sound generator for this script."""
+        return sound_generator.SoundGenerator(self)
+
     def getSpeechGenerator(self):
         """Returns the speech generator for this script.
         """
@@ -224,6 +228,10 @@ class Script:
         """Returns the spellcheck support for this script."""
         return None
 
+    def getCaretNavigation(self):
+        """Returns the caret navigation support for this script."""
+        return None
+
     def getUtilities(self):
         """Returns the utilites for this script.
         """
@@ -240,10 +248,14 @@ class Script:
         return []
 
     def getStructuralNavigation(self):
-        """Returns the 'structural navigation' class for this script.
-        """
+        """Returns the 'structural navigation' class for this script."""
         types = self.getEnabledStructuralNavigationTypes()
-        return structural_navigation.StructuralNavigation(self, types)
+        enable = _settingsManager.getSetting('structuralNavigationEnabled')
+        return structural_navigation.StructuralNavigation(self, types, enable)
+
+    def getLiveRegionManager(self):
+        """Returns the live region support for this script."""
+        return None
 
     def useStructuralNavigationModel(self):
         """Returns True if we should use structural navigation. Most
@@ -299,26 +311,6 @@ class Script:
 
         _eventManager.deregisterScriptListeners(self)
 
-    def getSettings(self):
-        """Returns the settings associated with this script, regardless of
-        whether or not the script is active.
-        """
-
-        scriptSettings = settings
-        if orca_state.activeScript != self:
-            name = _scriptManager.getModuleName(self.app)
-            if name:
-                for package in _settingsManager.settingsPackages:
-                    name = package + "." + name
-                    try:
-                        module = importlib.import_module(name)
-                        imp.reload(module)
-                        scriptSettings = module.orca.settings
-                    except:
-                        pass
-
-        return scriptSettings
-
     # [[[WDW - There is a circular reference going on somewhere (see
     # bug 333168).  In the presence of this reference, the existence
     # of a __del__ method prevents the garbage collector from
@@ -344,8 +336,8 @@ class Script:
         try:
             role = event.source.getRole()
         except (LookupError, RuntimeError):
-            msg = 'script.processObjectEvent: Error getting role'
-            debug.println(debug.LEVEL_FINE, msg)
+            msg = 'ERROR: Exception getting role for %s' % event.source
+            debug.println(debug.LEVEL_INFO, msg, True)
             return
 
         # Check to see if we really want to process this event.
@@ -377,7 +369,7 @@ class Script:
         # to the order in which they were added.  So...we need to do something
         # different here.  Logged as bugzilla bug 319781.]]]
         #
-        for key in list(self.listeners.keys()):
+        for key in self.listeners.keys():
             if event.type.startswith(key):
                 self.listeners[key](event)
 
@@ -410,11 +402,20 @@ class Script:
              and event.detail1 == cachedEvent.detail1:
             skip = True
             reason = "more recent event of the same type in a different object"
+        elif event.type.endswith("system") and event.source == cachedEvent.source:
+            skip = True
+            reason = "more recent system event in the same object"
+        elif event.type.startswith("object:state-changed") \
+             and event.type == cachedEvent.type \
+             and event.source == cachedEvent.source \
+             and event.detail1 == cachedEvent.detail1:
+            skip = True
+            reason = "appears to be duplicate state-changed event"
 
         if skip:
-            debug.println(debug.LEVEL_FINE,
-                          "script.skipObjectEvent: skipped due to %s" \
-                          % reason)
+            eventDetails = '           %s' % str(cachedEvent).replace('\t', ' ' * 11)
+            msg = 'SCRIPT: Skipping object event due to %s\n%s' % (reason, eventDetails)
+            debug.println(debug.LEVEL_INFO, msg, True)
 
         return skip
 
@@ -435,9 +436,8 @@ class Script:
             from gi.repository import Gdk
             keyboardEvent.keyval_name = Gdk.keyval_name(keyboardEvent.id)
         except:
-            debug.println(debug.LEVEL_FINE,
-                          "Could not obtain keyval_name for id: %d" \
-                          % keyboardEvent.id)
+            msg = 'ERROR: Exception getting keyval_name for id: %d' % keyboardEvent.id
+            debug.println(debug.LEVEL_INFO, msg, True)
 
     def consumesKeyboardEvent(self, keyboardEvent):
         """Called when a key is pressed on the keyboard.
@@ -465,7 +465,7 @@ class Script:
                 if consumes:
                     self._lastCommandWasStructNav = True
             else:
-                consumes = handler != None
+                consumes = handler is not None
         if not consumes:
             handler = self.keyBindings.getInputHandler(keyboardEvent)
             if handler \
@@ -474,55 +474,9 @@ class Script:
                 if consumes:
                     self._lastCommandWasStructNav = True
             else:
-                consumes = handler != None
+                consumes = handler is not None
         return consumes
 
-    def processKeyboardEvent(self, keyboardEvent):
-        """Processes the given keyboard event.
-
-        This method will primarily use the keybindings field of this
-        script instance see if this script has an interest in the
-        event.
-
-        NOTE: there is latent, but unsupported, logic for allowing
-        the user's user-settings.py file to extend and/or override
-        the keybindings for a script.
-
-        Arguments:
-        - keyboardEvent: an instance of input_event.KeyboardEvent
-        """
-
-        # We'll annotate the event with a reference to this script.
-        # This will allow external scripts to muck with the script
-        # instance if they wish.
-        #
-        keyboardEvent.script = self
-
-        # We'll let the user keybindings take precedence.  First, we'll
-        # check to see if they have keybindings specific for the particular
-        # application, then we'll check to see if they have any default
-        # bindings to use.
-        #
-        # [[[TODO: WDW - for performance, these bindings should probably
-        # be conflated at initialization time.]]]
-        #
-        user_bindings = None
-
-        user_bindings_map = settings.keyBindingsMap
-        if self.__module__ in user_bindings_map:
-            user_bindings = user_bindings_map[self.__module__]
-        elif "default" in user_bindings_map:
-            user_bindings = user_bindings_map["default"]
-
-        consumed = False
-        if user_bindings:
-            consumed = user_bindings.consumeKeyboardEvent(self,
-                                                          keyboardEvent)
-        if not consumed:
-            consumed = self.keyBindings.consumeKeyboardEvent(self,
-                                                             keyboardEvent)
-        return consumed
-
     def consumesBrailleEvent(self, brailleEvent):
         """Called when a key is pressed on the braille display.
 
